import { FuseBox, QuantumPlugin, EnvPlugin, SourceMapPlainJsPlugin } from "fuse-box";
import { task, src } from "fuse-box/sparky";
import fs from 'fs';

const isProduction = process.env.NODE_ENV === 'production';
const TARGET = process.env.TARGET;
const BROWSER = !TARGET || TARGET.startsWith('browser');
const prefix = (BROWSER ? '.browser' : '') + (isProduction ? '.min' : '')

task('replace-global', () => {
  return src('dist/*.js').file('*.browser.js', (file) => {
    const contents = fs.readFileSync(file.filepath, 'utf8')
      .replace('var global = window', 'var global = self;')
      .replace(/window\[['"]SonicDB['"]\]=(.*)\(0\)/, '$1(0)');
    return fs.writeFileSync(file.filepath, contents, 'utf8')
  });
});

task("build", async () => {
  const fuse = FuseBox.init({
    homeDir: "src",
    output: "dist/$name.js",
    sourceMaps: { sourceRoot: 'src', inline: true },
    target: TARGET || "browser@es2015",
    globals: { sonicdb: "SonicDB" },
    useTypescriptCompiler: true,
    plugins: [
      QuantumPlugin({
        replaceTypeOf: false,
        containedAPI: true,
        bakeApiIntoBundle: "sonicdb" + prefix,
        uglify: isProduction,
        treeshake: isProduction,
        removeExportsInterop: false,
      }),
      EnvPlugin({
        BROWSER,
        NODE_ENV: process.env.NODE_ENV
      }),
      SourceMapPlainJsPlugin()
    ]
  });

  fuse.bundle("sonicdb" + prefix)
    .instructions((!BROWSER) ? "> [index.ts]" : "> index.ts");

  await fuse.run();
});

task("default", ["build", 'replace-global']);
